import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { RunOnceScheduler, DeferredPromise } from 'vscode/vscode/vs/base/common/async';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { CancellationError } from 'vscode/vscode/vs/base/common/errors';
import { Iterable } from 'vscode/vscode/vs/base/common/iterator';
import { Disposable, DisposableMap, DisposableStore, MutableDisposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { ResourceSet } from 'vscode/vscode/vs/base/common/map';
import { localize } from 'vscode/vscode/vs/nls';
import { Action2, MenuId, registerAction2 } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { ILabelService } from 'vscode/vscode/vs/platform/label/common/label.service';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';
import { IFilesConfigurationService } from 'vscode/vscode/vs/workbench/services/filesConfiguration/common/filesConfigurationService.service';
import { ITextFileService } from 'vscode/vscode/vs/workbench/services/textfile/common/textfiles.service';
import { ChatAgentLocation } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents';
import { IChatAgentService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents.service';
import { CONTEXT_CHAT_REQUEST_IN_PROGRESS, CONTEXT_CHAT_LOCATION, CONTEXT_IN_CHAT_INPUT } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatContextKeys';
import { hasUndecidedChatEditingResourceContextKey, CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME, applyingChatEditsFailedContextKey, hasAppliedChatEditsContextKey } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatEditingService';
import { IChatEditingService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatEditingService.service';

var ChatEditorSaving_1;
let ChatEditorSaving = class ChatEditorSaving extends Disposable {
    static { ChatEditorSaving_1 = this; }
    static { this.ID = 'workbench.chat.editorSaving'; }
    static { this._config = 'chat.editing.alwaysSaveWithGeneratedChanges'; }
    constructor(configService, chatEditingService, chatAgentService, textFileService, labelService, dialogService, _fileConfigService) {
        super();
        this._fileConfigService = _fileConfigService;
        this._sessionStore = this._store.add(( (new DisposableMap())));
        const store = this._store.add(( (new DisposableStore())));
        const update = () => {
            store.clear();
            const alwaysSave = configService.getValue(ChatEditorSaving_1._config);
            if (alwaysSave) {
                return;
            }
            if (chatEditingService.currentEditingSession) {
                this._handleNewEditingSession(chatEditingService.currentEditingSession, store);
            }
            const saveJobs = new (class {
                constructor() {
                    this._soon = ( (new RunOnceScheduler(() => this._prompt(), 0)));
                    this._uris = ( (new ResourceSet()));
                }
                add(uri) {
                    this._uris.add(uri);
                    this._soon.schedule();
                    this._deferred ??= ( (new DeferredPromise()));
                    return this._deferred.p;
                }
                async _prompt() {
                    const alwaysSave = configService.getValue(ChatEditorSaving_1._config);
                    if (alwaysSave) {
                        return;
                    }
                    const uri = Iterable.first(this._uris);
                    if (!uri) {
                        return;
                    }
                    const agentName = chatAgentService.getDefaultAgent(ChatAgentLocation.EditingSession)?.fullName ?? ( localize(7595, "chat"));
                    const filelabel = labelService.getUriBasenameLabel(uri);
                    const message = this._uris.size === 1
                        ? ( localize(
                        7596,
                        "Do you want to save the changes {0} made in {1}?",
                        agentName,
                        filelabel
                    ))
                        : ( localize(
                        7597,
                        "Do you want to save the changes {0} made to {1} files?",
                        agentName,
                        this._uris.size
                    ));
                    const result = await dialogService.confirm({
                        message,
                        detail: ( localize(
                            7598,
                            "AI-generated changes may be incorrect and should be reviewed before saving.",
                            agentName
                        )),
                        primaryButton: ( localize(7599, "Save")),
                        cancelButton: ( localize(7600, "Cancel")),
                        checkbox: {
                            label: ( localize(7601, "Always save with AI-generated changes without asking")),
                            checked: false
                        }
                    });
                    this._uris.clear();
                    if (result.confirmed && result.checkboxChecked) {
                        await configService.updateValue(ChatEditorSaving_1._config, true);
                    }
                    if (!result.confirmed) {
                        this._deferred?.error(( (new CancellationError())));
                    }
                    else {
                        this._deferred?.complete();
                    }
                    this._deferred = undefined;
                }
            });
            store.add(chatEditingService.onDidCreateEditingSession(e => this._handleNewEditingSession(e, store)));
            store.add(textFileService.files.addSaveParticipant({
                participate: async (workingCopy, context, progress, token) => {
                    if (context.reason !== 1 ) {
                        return;
                    }
                    const session = chatEditingService.getEditingSession(workingCopy.resource);
                    if (!session) {
                        return;
                    }
                    if (!session.entries.get().find(e => e.state.get() === 0  && ( (e.modifiedURI.toString())) === ( (workingCopy.resource.toString())))) {
                        return;
                    }
                    return saveJobs.add(workingCopy.resource);
                }
            }));
        };
        configService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration(ChatEditorSaving_1._config)) {
                update();
            }
        });
        update();
    }
    _handleNewEditingSession(session, container) {
        const store = ( (new DisposableStore()));
        container.add(store);
        const saveConfig = store.add(( (new MutableDisposable())));
        const update = () => {
            const store = ( (new DisposableStore()));
            const entries = session.entries.get();
            for (const entry of entries) {
                if (entry.state.get() === 0 ) {
                    store.add(this._fileConfigService.disableAutoSave(entry.modifiedURI));
                }
            }
            saveConfig.value = store;
        };
        update();
        this._sessionStore.set(session, store);
        store.add(session.onDidChange(() => {
            update();
        }));
        store.add(session.onDidDispose(() => {
            store.dispose();
            container.delete(store);
        }));
    }
};
ChatEditorSaving = ChatEditorSaving_1 = ( (__decorate([
    ( (__param(0, IConfigurationService))),
    ( (__param(1, IChatEditingService))),
    ( (__param(2, IChatAgentService))),
    ( (__param(3, ITextFileService))),
    ( (__param(4, ILabelService))),
    ( (__param(5, IDialogService))),
    ( (__param(6, IFilesConfigurationService)))
], ChatEditorSaving)));
class ChatEditingSaveAllAction extends Action2 {
    static { this.ID = 'chatEditing.saveAllFiles'; }
    static { this.LABEL = ( localize(7602, 'Save All')); }
    constructor() {
        super({
            id: ChatEditingSaveAllAction.ID,
            title: ChatEditingSaveAllAction.LABEL,
            tooltip: ChatEditingSaveAllAction.LABEL,
            precondition: ( (ContextKeyExpr.and(
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                hasUndecidedChatEditingResourceContextKey
            ))),
            icon: Codicon.saveAll,
            menu: [
                {
                    when: ( (ContextKeyExpr.equals('resourceScheme', CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME))),
                    id: MenuId.EditorTitle,
                    order: 2,
                    group: 'navigation',
                },
                {
                    id: MenuId.ChatEditingWidgetToolbar,
                    group: 'navigation',
                    order: 2,
                    when: ( (ContextKeyExpr.and(
                         (applyingChatEditsFailedContextKey.negate()),
                         (ContextKeyExpr.or(
                            hasUndecidedChatEditingResourceContextKey,
                             (hasAppliedChatEditsContextKey.negate())
                        )),
                         (ContextKeyExpr.notEquals('config.files.autoSave', 'off')),
                         (ContextKeyExpr.equals(`config.${ChatEditorSaving._config}`, false)),
                         (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession))
                    )))
                }
            ],
            keybinding: {
                primary: 2048  | 49 ,
                when: ( (ContextKeyExpr.and(
                     (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                    hasUndecidedChatEditingResourceContextKey,
                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                    CONTEXT_IN_CHAT_INPUT
                ))),
                weight: 200 ,
            },
        });
    }
    async run(accessor, ...args) {
        const chatEditingService = accessor.get(IChatEditingService);
        const editorService = accessor.get(IEditorService);
        const configService = accessor.get(IConfigurationService);
        const chatAgentService = accessor.get(IChatAgentService);
        const dialogService = accessor.get(IDialogService);
        const labelService = accessor.get(ILabelService);
        const currentEditingSession = chatEditingService.currentEditingSession;
        if (!currentEditingSession) {
            return;
        }
        const editors = [];
        for (const modifiedFileEntry of currentEditingSession.entries.get()) {
            if (modifiedFileEntry.state.get() === 0 ) {
                const modifiedFile = modifiedFileEntry.modifiedURI;
                const matchingEditors = editorService.findEditors(modifiedFile);
                if (matchingEditors.length === 0) {
                    continue;
                }
                const matchingEditor = matchingEditors[0];
                if (matchingEditor.editor.isDirty()) {
                    editors.push(matchingEditor);
                }
            }
        }
        if (editors.length === 0) {
            return;
        }
        const alwaysSave = configService.getValue(ChatEditorSaving._config);
        if (!alwaysSave) {
            const agentName = chatAgentService.getDefaultAgent(ChatAgentLocation.EditingSession)?.fullName;
            let message;
            if (editors.length === 1) {
                const resource = editors[0].editor.resource;
                if (resource) {
                    const filelabel = labelService.getUriBasenameLabel(resource);
                    message = agentName
                        ? ( localize(
                        7603,
                        "Do you want to save the changes {0} made in {1}?",
                        agentName,
                        filelabel
                    ))
                        : ( localize(7604, "Do you want to save the changes chat made in {0}?", filelabel));
                }
                else {
                    message = agentName
                        ? ( localize(7605, "Do you want to save the changes {0} made in 1 file?", agentName))
                        : ( localize(7606, "Do you want to save the changes chat made in 1 file?"));
                }
            }
            else {
                message = agentName
                    ? ( localize(
                    7607,
                    "Do you want to save the changes {0} made in {1} files?",
                    agentName,
                    editors.length
                ))
                    : ( localize(
                    7608,
                    "Do you want to save the changes chat made in {0} files?",
                    editors.length
                ));
            }
            const result = await dialogService.confirm({
                message,
                detail: ( localize(
                    7598,
                    "AI-generated changes may be incorrect and should be reviewed before saving.",
                    agentName
                )),
                primaryButton: ( localize(7609, "Save All")),
                cancelButton: ( localize(7600, "Cancel")),
                checkbox: {
                    label: ( localize(7601, "Always save with AI-generated changes without asking")),
                    checked: false
                }
            });
            if (!result.confirmed) {
                return;
            }
            if (result.checkboxChecked) {
                await configService.updateValue(ChatEditorSaving._config, true);
            }
        }
        await editorService.save(editors, { reason: 1 , skipSaveParticipants: true });
    }
}
registerAction2(ChatEditingSaveAllAction);

export { ChatEditingSaveAllAction, ChatEditorSaving };
